Package: samr
Title: SAM: Significance Analysis of Microarrays
Version: 3.0.1
Description: Significance Analysis of Microarrays for differential expression analysis, RNAseq data and related problems.
Imports: impute, matrixStats, shiny, shinyFiles, openxlsx, GSA
License: LGPL
Packaged: 2026-04-10 06:51:13 UTC; hornik
Repository: CRAN
Date/Publication: 2026-04-10 07:11:47 UTC
NeedsCompilation: yes
Authors@R: c(person(given = "Rob",
 	            family = "Tibshirani",
                    role = c("aut", "cre"),
		    email = "tibs@stanford.edu"),
             person(given = c("Michael", "J."),
                    family = "Seo",
                    role = "aut"),
             person(given = "G.",
                    family = "Chu",
                    role = "aut"),
             person(given = "Balasubramanian",
                    family = "Narasimhan",
                    role = "aut"),
             person(given = "Jun",
                    family = "Li",
                    role = "aut"))
Author: Rob Tibshirani [aut, cre],
  Michael J. Seo [aut],
  G. Chu [aut],
  Balasubramanian Narasimhan [aut],
  Jun Li [aut]
Maintainer: Rob Tibshirani <tibs@stanford.edu>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 21:05:04 UTC; unix
Archs: samr.so.dSYM
