# Web-enabled and Cross-platform SAM via Shiny

This version of SAM provided adds a web interface using Shiny.

Install the current version of the package, after removing any
older versions you may have already installed. _Later, when this
package makes it to CRAN, you will install it the usual way_.

The web interface is invoked via:

```
library(samr)
runSAM()
```

That will bring up a browser window with a user interface. More
details are provided in the manual `sam.pdf` in this directory

Note that users should use Firefox or Chrome as the _default_browser
on Windows: IE will not work.

Michael Seo
